<?xml version="1.0"?>

<st-source>
<time-stamp>From VisualWorks®, 7.3.1 of April 20, 2005 on August 27, 2010 at 4:36:58 pm</time-stamp>
<!-- Changes from: PV-Datachannel(Green.258.7.18,bbuxton)  to: PV-Datachannel (Green.258.7.19,bbuxton)+ -->


<methods>
<class-id>PROVISO.UBA.AbstractSchema</class-id> <category>accessing</category>

<body package="PVAdaptor-Schema" selector="onRead">onRead
	&lt;getterNamed: #onRead comment: 'Return the function to call after reading the next record for the input schema'&gt;
	^onRead</body>

<body package="PVAdaptor-Schema" selector="rootSchema">rootSchema
	&lt;getterNamed: #rootSchema comment: 'Return the top most input schema'&gt;
	^parent
		ifNil: [self]
		ifNotNil: [:myParent | myParent rootSchema]</body>
</methods>

<methods>
<class-id>PROVISO.UBA.FecordSchema</class-id> <category>events</category>

<body package="PVAdaptor-Schema" selector="triggerOnReadWith:">triggerOnReadWith: aFieldOrRecord
	| inputDescriptor |
	Smee.SmeeEnvironment current eventValue: aFieldOrRecord.
	inputDescriptor := self rootSchema currentObject.
	^self onRead valueWithThis: self with: aFieldOrRecord with: inputDescriptor.</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputDescriptorScheduler</class-id> <category>processing</category>

<body package="PVAdaptor-Schema" selector="processAll:">processAll: inputs 
	" If the schema is not a multifile join, there should ordinarily be only one file to process at a time.
	If multiple files are found, it is probably a mistake (i.e. script did not bother to set a timestamp.
	In this case sort files by local name as a meek protection (increases likelihood files will be processed
	in desired order. Quite frankly, the only reason we bother is because there are some sloppy test cases
	that don't set descriptor's timestamp "
	
	| realInputs |
	realInputs := 
		(inputs size &gt; 1 and: [schema multiFileJoin not]) 
			ifTrue: 
				[self system 
					logWarning: #MULTIPLE_FILES_PER_PERIOD_DETECTED
					messageId: #DC10428
					text: 
						'Multiple files per period ' , 
							inputs anyOne timestamp smeeToPrimitiveNumber pvOpportunisticPrintString , 
								' are found -- this should not happen. Please check the design script and data feed mechanism'.
				(inputs asSortedCollection: [:i1 :i2 | i1 localName &lt; i2 localName]) asOrderedCollection]
			ifFalse: [inputs].
	realInputs do: [:each | each markScheduled].
	schema process: realInputs.</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>parsing</category>

<body package="PVAdaptor-Schema" selector="process:">process: allInputs

	^handlerBlock value: allInputs</body>

<body package="PVAdaptor-Schema" selector="processInputDescriptor:">processInputDescriptor: aDescriptor
	
	^self process: (Array with: aDescriptor)</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchema</class-id> <category>initialize-release</category>

<body package="PVAdaptor-Schema" selector="initialize">initialize
	
	super initialize.
	environment := Smee.SmeeEnvironment current.
	recordNumber := 0.
	scannedInputs := SmeeArray new.
	scannedInputs1 := #().
	scannedInputs2 := #().
	localEnvironment := Dictionary new.
	precedence := 0.
	inputAgeLimit := 168.		"7 days expressed in hours"
	errorFileCount := 0.
	errorFileLimit := Infinity positive.
	deleteOnAcquire := false.
	retryLimit := 5.
	acquireURI := true.
	ignoreProcessingErrors := false.
	rejectedRecordCount := 0.
	self heartbeatPeriod: 1.		" Generate heartbeats every second "
	handlerBlock := [:allInputs | self adaptor processInputDescriptor: allInputs]</body>
</methods>



<methods>
<class-id>PROVISO.PvSAMJmsInterface</class-id> <category>process management</category>

<body package="SAM-Interface" selector="processInventoryChangeEvents:">processInventoryChangeEvents: testBlock 
	
	self currentApplication server environmentDo: 
		[[testBlock value] 
			whileTrue: [self processNextInventoryChangeEvent]]</body>
</methods>


<methods>
<class-id>PROVISO.ForeignDomainMethod</class-id> <category>invocation</category>

<body package="SAM-DomainModel" selector="acquireFile:">acquireFile: aURI 
	
	| localURI |
	self system 
		logDebugFour: #ACQUIRE_FILE
		text: 'Acquiring file ' , aURI asEncryptedString.
	localURI := self exportFilenameLocalDirectory , aURI tail.
	self waitFor: aURI.
	self 
		copy: aURI
		to: localURI.
	self system 
		logDebugFour: #ACQUIRE_FILE
		text: 'Successfully acquired file into ' , localURI printString.
	^localURI</body>
</methods>

<methods>
<class-id>PROVISO.ForeignDomainMethod</class-id> <category>private</category>

<body package="SAM-DomainModel" selector="copy:to:">copy: aURI to: localURI 
	
	| delay copyAttempts |
	delay := 500.
	copyAttempts := 0.
	[(copyAttempts := copyAttempts + 1) &gt; 5 
		ifTrue: 
			[^aURI fileSize isZero 
				ifTrue: 
					[self system 
						logDebugFour: #ACQUIRE_FILE
						text: 'Acquired zero length ' , localURI printString]
				ifFalse: [self error: 'copy attempt limit reached']].
	aURI copyTo: localURI.
	localURI fileSize = aURI fileSize and: [aURI fileSize isZero not]] 
		whileFalse: [(self timeFacade delayForMilliseconds: (delay := delay * 2)) wait]</body>

<body package="SAM-DomainModel" selector="waitFor:">waitFor: aURI 
	
	"Wait up to 5 minutes for an export file to appear"
	| delay deadline |
	deadline := Time totalSeconds + 300.
	delay := 500.
	[aURI exists] 
		whileFalse: 
			[Time totalSeconds &gt; deadline 
				ifTrue: [self error: 'export file not found'].
			(self timeFacade delayForMilliseconds: (delay := delay * 2)) wait]</body>
</methods>

<methods>
<class-id>PROVISO.ForeignDomainObject</class-id> <category>smee</category>

<body package="SAM-DomainModel" selector="smeeAllEnumerablePropertiesDo:">smeeAllEnumerablePropertiesDo: aBlock 
	
	self properties associationsDo: aBlock</body>

<body package="SAM-DomainModel" selector="smeeAllPropertiesWhere:notIn:">smeeAllPropertiesWhere: aBlock notIn: aDictionary 
	
	| property |
	self properties associationsDo: 
		[:eachAssociation | 
		property := 
			SmeeProperty 
				key: eachAssociation key
				value: eachAssociation value.
		((aBlock value: property) and: [aDictionary includesKey: eachAssociation key]) 
			ifFalse: [aDictionary add: property]].
	super 
		smeeAllPropertiesWhere: aBlock
		notIn: aDictionary.
	^aDictionary</body>
</methods>


<methods>
<class-id>PROVISO.UBA.InputSchemaProcessor</class-id> <category>building</category>

<body package="NGUBA-Application" selector="connectAsInputToMultiInputOperator:builder:">connectAsInputToMultiInputOperator: aClientOperator builder: aPlanBuilder 
	" Second leg of dispatch. We now know that the client is a multi-input operator, 
	now dispatch based on the input type -- in this case input schema "
	
	self schema handlerBlock: [:allInputs | self queue nextPut: allInputs].
	self startAsDaemon: true.
	^aPlanBuilder 
		connectInputSchema: self
		toMultiInputOperator: aClientOperator</body>

<body package="NGUBA-Application" selector="connectAsInputToSingleInputOperator:builder:">connectAsInputToSingleInputOperator: aClientOperator builder: aPlanBuilder 
	" Second leg of dispatch. We now know that the client is a single-input operator, 
	now dispatch based on the input type -- in this case input schema "
	
	self schema handlerBlock: [:allInputs | self process: allInputs].
	self startAsDaemon: false.		" Processing is synchronous "
	^aPlanBuilder 
		connectInputSchema: self
		toSingleInputOperator: aClientOperator</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchemaProcessor</class-id> <category>private</category>

<body package="NGUBA-Application" selector="descriptorNamed:">descriptorNamed: aName

	^self schema inputDescriptors at: aName ifAbsent: [nil]</body>

<body package="NGUBA-Application" selector="process:ifValidDo:">process: anInputDescriptor ifValidDo: aBlock 
	
	| currentDesc |
	currentDesc := self descriptorNamed: anInputDescriptor name.
	currentDesc ifNil: [currentDesc := anInputDescriptor].
	currentDesc alreadyProcessed 
		ifTrue: [^self].
	currentDesc ignore 
		ifTrue: 
			[currentDesc doNotProcess.
			^self].
	(self schema isValidInputDescriptor: currentDesc) 
		ifFalse: 
			[currentDesc uriInDoneDirectory exists 
				ifTrue: [self moveFileToDoFor: currentDesc].
			(self schema isValidInputDescriptor: currentDesc) 
				ifFalse: 
					[self schema logInputDescriptorMissingFor: currentDesc.
					currentDesc doNotProcess.
					^self]].
	(self hasValidTimestampForDescriptor: currentDesc) 
		ifFalse: [self logNoTimestampForDescriptor: currentDesc].
	self descriptor: currentDesc.
	currentDesc adaptor: self.
	[aBlock value: currentDesc] 
		ensure: 
			[self descriptor: nil.
			currentDesc adaptor: nil]</body>

<body package="NGUBA-Application" selector="processLoop">processLoop

	self process: self queue next</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchemaProcessor</class-id> <category>obsolete</category>

<body package="NGUBA-Application" selector="connectToSchema:asynchronously:">connectToSchema: anInputSchema asynchronously: aBoolean

	self schema: anInputSchema.
	self traceHeartbeats: anInputSchema traceHeartbeats.
	anInputSchema handlerBlock:
		(aBoolean 
			ifTrue: [ [:allInputs | self queue nextPut: allInputs] ]
			ifFalse: [ [:allInputs | self processInputDescriptors: allInputs] ]).
	self startAsDaemon: aBoolean.
	self start</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchemaProcessor</class-id> <category>back end callbacks</category>

<body package="NGUBA-Application" selector="process:">process: inputDescriptors

	self processInputDescriptors: inputDescriptors</body>

<body package="NGUBA-Application" selector="processInputDescriptors:">processInputDescriptors: descriptors 
	
	descriptors do: [:each | self processInputDescriptor: each]</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchemaProcessorStream class</class-id> <category>public</category>

<body package="NGUBA-Application" selector="for:using:">for: anInputSchemaMultiJoinProcessor using: anInputDescriptorName

	^self basicNew
		initializeFor: anInputSchemaMultiJoinProcessor using: anInputDescriptorName</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchemaMultiJoinProcessor</class-id> <category>public</category>

<body package="NGUBA-Application" selector="processInputDescriptor:">processInputDescriptor: anInputDescriptor 
	
	^self processInputDescriptors: (Array with: anInputDescriptor)</body>

<body package="NGUBA-Application" selector="processInputDescriptors:">processInputDescriptors: descriptors 
	
	| stream toProcess |
	toProcess := descriptors reject: [:each | self shouldIgnore: each].
	toProcess isEmpty 
		ifTrue: [^self].
	toProcess do: 
		[:eachDescriptor | 
		(self hasValidTimestampForDescriptor: eachDescriptor) 
			ifFalse: [self logNoTimestampForDescriptor: eachDescriptor].
		stream := self newStreamFor: eachDescriptor name.
		descriptorStreams at: eachDescriptor name put: stream.
		mergeStream addStream: stream].
	self processAll</body>

<body package="NGUBA-Application" selector="produceRecord:">produceRecord: aRecord 
	
	| stream |
	stream := 
		descriptorStreams 
			at: descriptor name
			ifAbsent: [self error: 'no descriptor'].
	stream nextPut: 
		(RecordDecorator 
			on: aRecord
			from: stream descriptor)</body>
</methods>

<methods>
<class-id>PROVISO.UBA.InputSchemaMultiJoinProcessor</class-id> <category>private</category>

<body package="NGUBA-Application" selector="descriptorsReadyForProcessing">descriptorsReadyForProcessing
	
	^(descriptorStreams keys asOrderedCollection collect: 
		[:each | 
		self descriptorNamed: each]) reject: [:each | each isNil]</body>

<body package="NGUBA-Application" selector="finishParsingFor:">finishParsingFor: anInputDescriptor 
	
	finishedDescriptors add: (self descriptorNamed: anInputDescriptor name).
	descriptorStreams removeKey: anInputDescriptor name</body>

<body package="NGUBA-Application" selector="newStreamFor:">newStreamFor: anInputDescriptorName

	^InputSchemaProcessorStream for: self using: anInputDescriptorName</body>
</methods>


</st-source>
